/*
 * This file is part of libslab.
 *
 * Copyright (c) 2006 Novell, Inc.
 *
 * Libslab is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * Libslab is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libslab; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */


#ifndef __SLAB_H__
#define __SLAB_H__

#include "app-resizer.h"
#include "app-shell.h"
#include "application-tile.h"
#include "bookmark-agent.h"
#include "double-click-detector.h"
#include "mate-utils.h"
#include "libslab-utils.h"
#include "nameplate-tile.h"
#include "search-bar.h"
#include "search-context-picker.h"
#include "shell-window.h"
#include "slab-mate-util.h"
#include "slab-section.h"
#include "tile.h"

#endif /* __SLAB_H__ */

